<?php
/* Copyright (C) 2020      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


class ActionsPdfDownloadNotifier
{
    /**
     * @var DoliDB Database handler.
     */
    public $db;
    /**
     * @var string Error
     */
    public $error = '';
    /**
     * @var array Errors
     */
    public $errors = array();

    /**
     * @var array Hook results. Propagated to $hookmanager->resArray for later reuse
     */
    public $results = array();

    /**
     * @var string 	String displayed by executeHook() immediately after return
     */
    public $resprints;

	/**
	 * @var string 	Type email template
	 */
	public 	$type_email_template = 'pdn_notify_send';


	/**
     * Constructor
     *
     * @param        DoliDB $_db Database handler
     */
    public function __construct($_db)
    {
        global $db;
        $this->db = is_object($_db) ? $_db : $db;
    }

	/**
	 * Overloading the downloadDocument function : replacing the parent's function with the one below
	 *
	 * @param   array()			$parameters		Hook metadatas (context, etc...)
	 * @param   CommonObject	&$object		The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string			&$action		Current action (if set). Generally create or edit or null
	 * @param   HookManager		$hookmanager	Hook manager propagated to allow calling another hook
	 * @return  int                             < 0 on error, 0 on success, 1 to replace standard code
	 */
	function downloadDocument($parameters, &$object, &$action, $hookmanager)
	{
		global $conf;

		if (empty($conf->pdfdownloadnotifier->enabled)) return 0;

		$ecmfile = $parameters['ecmfile'];
		if (is_object($ecmfile)) {
			$emailNotifyConstName = 'PDFDOWNLOADNOTIFIER_NOTIFY_BY_EMAIL_' . strtoupper($ecmfile->src_object_type);
			$eventNotifyConstName = 'PDFDOWNLOADNOTIFIER_NOTIFY_BY_EVENT_' . strtoupper($ecmfile->src_object_type);

			if (!empty($conf->global->$emailNotifyConstName) || !empty($conf->global->$eventNotifyConstName)) {
				require_once DOL_DOCUMENT_ROOT . '/core/lib/functions2.lib.php';
				$obj = fetchObjectByElement($ecmfile->src_object_id, $ecmfile->src_object_type);
				if (is_object($obj) && $obj->id > 0) {
					$download_filename = $ecmfile->filename;
					$download_filepath = !empty($ecmfile->filepath) ? DOL_DATA_ROOT . '/' . $ecmfile->filepath . '/'.$ecmfile->filename : '';
					$download_from_ip = !empty($_SERVER["REMOTE_ADDR"]) ? $_SERVER["REMOTE_ADDR"] : 'N/A';
					$download_from_user_agent = !empty($_SERVER["HTTP_USER_AGENT"]) ? $_SERVER["HTTP_USER_AGENT"] : 'N/A';
					$download_date = dol_now();

					$this->error = '';
					$this->errors = array();
					$result = 0;
					if (!empty($conf->global->$emailNotifyConstName)) $result = $this->_notify_by_mail($obj, $download_filename, $download_filepath, $download_from_ip, $download_from_user_agent, $download_date);
					if (!empty($conf->global->$eventNotifyConstName)) $result = $this->_notify_by_event($obj, $download_filename, $download_from_ip, $download_from_user_agent, $download_date);
					if ($result < 0) {
						$errors = $this->error . (is_array($this->errors) ? (!empty($this->error) ? ', ' : '') . join('; ', $this->errors) : '');
						dol_syslog(__METHOD__ . " - Errors when notify by event : " . $errors);
					}
				}
			}
		}

		return 0;
	}

	/**
	 * Overloading the emailElementlist function : replacing the parent's function with the one below
	 *
	 * @param   array()			$parameters		Hook metadatas (context, etc...)
	 * @param   CommonObject	&$object		The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string			&$action		Current action (if set). Generally create or edit or null
	 * @param   HookManager		$hookmanager	Hook manager propagated to allow calling another hook
	 * @return  int                             < 0 on error, 0 on success, 1 to replace standard code
	 */
	function emailElementlist($parameters, &$object, &$action, $hookmanager)
	{
		global $conf, $langs;

		if (empty($conf->pdfdownloadnotifier->enabled)) return 0;

		$langs->load('pdfdownloadnotifier@pdfdownloadnotifier');
		$this->results[$this->type_email_template] = $langs->trans('PdfDownloadNotifierMailToSendDownloadNotification');

		return 0;
	}

	/**
	 *  Get element list
	 *
	 * @return array|int		<0 if KO or Array with the list of element array(element_type => element_label, ...)
	 */
	public function getElementList()
	{
		global $hookmanager;

		$element_list = array(
			'propal' => array('label' => 'Proposal', 'langs' => 'propal'),
			'commande' => array('label' => 'Order', 'langs' => 'orders'),
			'facture' => array('label' => 'Invoice', 'langs' => 'bills'),
		);

		if (!is_object($hookmanager)) {
			include_once DOL_DOCUMENT_ROOT . '/core/class/hookmanager.class.php';
			$hookmanager = new HookManager($this->db);
		}
		$hookmanager->initHooks(array('pdfdownloadnotifierdao'));
		$parameters = array('element_list' => &$element_list);
		$reshook = $hookmanager->executeHooks('addElementList', $parameters); // Note that $action and $object may have been
		if ($reshook < 0) {
			$this->error = $hookmanager->error;
			$this->errors = $hookmanager->errors;
			$errors = $hookmanager->error . (is_array($hookmanager->errors) ? (!empty($hookmanager->error) ? ', ' : '') . join('; ', $hookmanager->errors) : '');
			dol_syslog(__METHOD__ . " - Errors when executing the hook : " . $errors);

			return -1;
		}

		return $element_list;
	}

	/**
	 *  Get email template list
	 *
	 * @return	array|int				<0 if KO or Array with the list of element array(key => label, ...)
	 */
	public function getEmailTemplateList()
	{
		global $user, $langs, $formmail;

		if (!is_object($formmail)) {
			require_once DOL_DOCUMENT_ROOT . '/core/class/html.formmail.class.php';
			$formmail = new FormMail($this->db);
		}

		$modelmail_array = array();
		$result = $formmail->fetchAllEMailTemplate($this->type_email_template, $user, $langs);
		if ($result < 0) {
			$this->error = $formmail->error;
			$this->errors = $formmail->errors;
			$errors = $formmail->error . (is_array($formmail->errors) ? (!empty($formmail->error) ? ', ' : '') . join('; ', $formmail->errors) : '');
			dol_syslog(__METHOD__ . " - Errors when fetch all email template : " . $errors);
			return -1;
		}

		if (is_array($formmail->lines_model)) {
			foreach ($formmail->lines_model as $line) {
				if (preg_match('/\((.*)\)/', $line->label, $reg)) {
					$modelmail_array[$line->id] = $langs->trans($reg[1]);        // langs->trans when label is __(xxx)__
				} else {
					$modelmail_array[$line->id] = $line->label;
				}
				if ($line->lang) $modelmail_array[$line->id] .= ' (' . $line->lang . ')';
				if ($line->private) $modelmail_array[$line->id] .= ' - ' . $langs->trans("Private");
			}
		}

		return $modelmail_array;
	}

	/**
	 *  Notify the download of the file by email
	 *
	 * @param CommonObject		$object				Handle object
	 * @param string			$filename			Filename downloaded
	 * @param string			$filepath			Filepath downloaded
	 * @param string			$from_ip			IP who have downloaded
	 * @param string			$from_user_agent	User agent who have downloaded
	 * @param int				$date				Date when is downloaded
	 * @return  int                         		<0 if KO, >0 if OK
	 */
	protected function _notify_by_mail(&$object, $filename, $filepath, $from_ip, $from_user_agent, $date)
	{
		global $conf, $langs, $user;

		$langs->load('pdfdownloadnotifier@pdfdownloadnotifier');

		$send_to = array();

		// Get defined type contact to send
		$contact_list = $object->liste_contact(-1, 'internal');
		if (!is_array($contact_list)) {
			$this->error = $object->error;
			$this->errors = $object->errors;
			dol_syslog(__METHOD__ . " - Errors when get internal contacts : " . $object->errorsToString());
			return -1;
		} elseif (!empty($contact_list)) {
			$element_prop = getElementProperties($object->element);
			if (is_array($element_prop) && $conf->{$element_prop['module']}->enabled) {
				$key = strtoupper($object->element);
				$emailNotifyContactTypeConstName = 'PDFDOWNLOADNOTIFIER_NOTIFY_CONTACT_TYPE_BY_EMAIL_' . $key;
				$contact_type_to_send = !empty($conf->global->$emailNotifyContactTypeConstName) ? explode(',', $conf->global->$emailNotifyContactTypeConstName) : array();

				if (!empty($contact_type_to_send)) {
					foreach ($contact_list as $contact_infos) {
						if (!in_array($contact_infos['fk_c_type_contact'], $contact_type_to_send) || empty($contact_infos['statuscontact'])) continue;
						$email = $contact_infos['email'];
						if (!empty($email)) {
							$send_to[$email] = $this->_formatEmail(dolGetFirstLastname($contact_infos['firstname'], $contact_infos['lastname']), $email);
						}
					}
				}
			}
		}

		// Get sales representatives to send if emails not found
		$result = $object->fetch_thirdparty();
		if (empty($send_to) && $result > 0 && method_exists($object->thirdparty, 'getSalesRepresentatives')) {
			$user_list = $object->thirdparty->getSalesRepresentatives($user);
			if (!is_array($user_list)) {
				$this->error = $this->db->lasterror();
				dol_syslog(__METHOD__ . " - Errors when get sales representatives : " . $this->error);
				return -1;
			}

			foreach ($user_list as $user_infos) {
				if (empty($user_infos['statut'])) continue;
				$email = $user_infos['email'];
				if (!empty($email)) {
					$send_to[$email] = $this->_formatEmail(dolGetFirstLastname($user_infos['firstname'], $user_infos['lastname']), $email);
				}
			}
		}

		// Get default email to send if emails not found
		if (empty($send_to) && !empty($conf->global->PDFDOWNLOADNOTIFIER_DEFAULT_SEND_TO)) {
			$send_to[] = $conf->global->PDFDOWNLOADNOTIFIER_DEFAULT_SEND_TO;
		}

		if (!empty($send_to)) {
			$send_to = implode(',', $send_to);

			$object->context['pdfdownloadnotifier_download_filename'] = $filename;
			$object->context['pdfdownloadnotifier_download_from_ip'] = $from_ip;
			$object->context['pdfdownloadnotifier_download_from_user_agent'] = $from_user_agent;
			$object->context['pdfdownloadnotifier_download_date'] = $date;

			require_once DOL_DOCUMENT_ROOT . '/core/lib/files.lib.php';
			require_once DOL_DOCUMENT_ROOT . '/core/class/html.formmail.class.php';
			$formmail = new FormMail($this->db);

			$langs->load('mails');
			$langs->load("commercial");
			if (!empty($conf->dolimail->enabled)) $langs->load("dolimail@dolimail");

			$trackid = 'pdn_' . $object->element . '_' . $object->id;
			$deliveryreceipt = 0;

			// from / send to / ...
			$from = $conf->global->MAIN_INFO_SOCIETE_NOM . ' <' . $conf->global->MAIN_INFO_SOCIETE_MAIL . '>';
			$sendtocc = '';
			$sendtobcc = '';

			// Define output language
			$outputlangs = $langs;
			$newlang = !empty($conf->global->MAIN_MULTILANGS) ? $object->thirdparty->default_lang : '';
			if (!empty($newlang)) {
				$outputlangs = new Translate('', $conf);
				$outputlangs->setDefaultLang($newlang);
				$outputlangs->loadLangs(array('commercial', 'bills', 'orders', 'contracts', 'members', 'propal', 'products', 'supplier_proposal', 'interventions'));
			}

			// Make substitution in email content
			$substitutionarray = getCommonSubstitutionArray($outputlangs, 0, null, $object);
			$substitutionarray['__CHECK_READ__'] = (is_object($object) && is_object($object->thirdparty)) ? '<img src="' . DOL_MAIN_URL_ROOT . '/public/emailing/mailing-read.php?tag=' . $object->thirdparty->tag . '&securitykey=' . urlencode(getDolGlobalString('MAILING_EMAIL_UNSUBSCRIBE_KEY')) . '" width="1" height="1" style="width:1px;height:1px" border="0"/>' : '';
			$substitutionarray['__PERSONALIZED__'] = '';    // deprecated
			$substitutionarray['__CONTACTCIVNAME__'] = '';
			$parameters = array('mode' => 'formemail');
			complete_substitutions_array($substitutionarray, $outputlangs, $object, $parameters);

			// Get email template
			if (!($conf->global->PDFDOWNLOADNOTIFIER_EMAIL_MODEL_USED_TO_NOTIFY_BY_EMAIL > 0)) {
				$this->error = $langs->trans('PdfDownloadNotifierErrorEmailTemplateNotConfigured');
				dol_syslog(__METHOD__ . " - Errors when get email template : " . $this->error);
				return -1;
			}
			$arraydefaultmessage = $formmail->getEMailTemplate($this->db, $this->type_email_template, $user, $outputlangs, $conf->global->PDFDOWNLOADNOTIFIER_EMAIL_MODEL_USED_TO_NOTIFY_BY_EMAIL);
			if (!is_array($arraydefaultmessage) && !is_object($arraydefaultmessage)) {
				$this->error = $formmail->error;
				$this->errors = $formmail->errors;
				$errors = $formmail->error . (is_array($formmail->errors) ? (!empty($formmail->error) ? ', ' : '') . join('; ', $formmail->errors) : '');
				dol_syslog(__METHOD__ . " - Errors when get email template : " . $errors);
				return -1;
			}

			// Define subject / message
			$message = str_replace('\n', "\n", is_array($arraydefaultmessage) ? $arraydefaultmessage['content'] : $arraydefaultmessage->content);
			// Deal with format differences between message and signature (text / HTML)
			if (dol_textishtml($message) && !dol_textishtml($substitutionarray['__USER_SIGNATURE__'])) {
				$substitutionarray['__USER_SIGNATURE__'] = dol_nl2br($substitutionarray['__USER_SIGNATURE__']);
			} else if (!dol_textishtml($message) && dol_textishtml($substitutionarray['__USER_SIGNATURE__'])) {
				$message = dol_nl2br($message);
			}

			$subject = make_substitutions(is_array($arraydefaultmessage) ? $arraydefaultmessage['topic'] : $arraydefaultmessage->topic, $substitutionarray);
			$message = make_substitutions($message, $substitutionarray);
			if (method_exists($object, 'makeSubstitution')) {
				$subject = $object->makeSubstitution($subject);
				$message = $object->makeSubstitution($message);
			}

			// Clean first \n and br (to avoid empty line when CONTACTCIVNAME is empty)
			$message = preg_replace("/^(<br>)+/", "", $message);
			$message = preg_replace("/^\n+/", "", $message);

			// Define $urlwithroot
			global $dolibarr_main_url_root;
			$urlwithouturlroot = preg_replace('/' . preg_quote(DOL_URL_ROOT, '/') . '$/i', '', trim($dolibarr_main_url_root));
			$urlwithroot = $urlwithouturlroot . DOL_URL_ROOT;        // This is to use external domain name found into config file
			//$urlwithroot=DOL_MAIN_URL_ROOT;					// This is to use same domain name than current
			// Make a change into HTML code to allow to include images from medias directory with an external reabable URL.
			// <img alt="" src="/dolibarr_dev/htdocs/viewimage.php?modulepart=medias&amp;entity=1&amp;file=image/ldestailleur_166x166.jpg" style="height:166px; width:166px" />
			// become
			// <img alt="" src="'.$urlwithroot.'viewimage.php?modulepart=medias&amp;entity=1&amp;file=image/ldestailleur_166x166.jpg" style="height:166px; width:166px" />
			$message = preg_replace('/(<img.*src=")[^\"]*viewimage\.php([^\"]*)modulepart=medias([^\"]*)file=([^\"]*)("[^\/]*\/>)/', '\1' . $urlwithroot . '/viewimage.php\2modulepart=medias\3file=\4\5', $message);

			// Attach file
			$formmail->trackid = $trackid;      // $trackid must be defined
			$formmail->clear_attached_files();
			if (!empty($filepath) && file_exists($filepath) && !empty(is_array($arraydefaultmessage) ? $arraydefaultmessage['joinfiles'] : $arraydefaultmessage->joinfiles)) {
				$formmail->add_attached_files($filepath, basename($filepath), dol_mimetype($filepath));
			}
			$attachedfiles = $formmail->get_attached_files();
			$filepath = $attachedfiles['paths'];
			$filename = $attachedfiles['names'];
			$mimetype = $attachedfiles['mimes'];

			// Send mail (substitutionarray must be done just before this)
			require_once DOL_DOCUMENT_ROOT . '/core/class/CMailFile.class.php';
			$sendcontext = 'standard';
			$mailfile = new CMailFile($subject, $send_to, $from, $message, $filepath, $mimetype, $filename, $sendtocc, $sendtobcc, $deliveryreceipt, -1, '', '', $trackid, '', $sendcontext);
			if ($mailfile->error) {
				$this->error = $mailfile->error;
				$this->errors = $mailfile->errors;
				$errors = $mailfile->error . (is_array($mailfile->errors) ? (!empty($mailfile->error) ? ', ' : '') . join('; ', $mailfile->errors) : '');
				dol_syslog(__METHOD__ . " - Errors when create mail file : " . $errors);
				return -1;
			} else {
				$result = $mailfile->sendfile();
				if (!$result) {
					$langs->load("other");
					if ($mailfile->error) {
						$this->errors[] = $langs->trans('ErrorFailedToSendMail', $from, $send_to);
						$this->errors[] = $mailfile->error;
					} else {
						$this->errors[] = ' No mail sent. Feature is disabled by option MAIN_DISABLE_ALL_MAILS';
					}
					$errors = (is_array($this->errors) ? join('; ', $this->errors) : '');
					dol_syslog(__METHOD__ . " - Errors when send mail file : " . $errors);
					return -1;
				}

				return 1;
			}
		}

		return 0;
	}

	/**
	 *  Notify the download of the file by event
	 *
	 * @param CommonObject		$object		Handle object
	 * @param string			$filename	Filename downloaded
	 * @param string			$from_ip	IP who have downloaded
	 * @param string			$from_user_agent	User agent who have downloaded
	 * @param int				$date		Date when is downloaded
	 * @return  int                         <0 if KO, >0 if OK
	 */
	protected function _notify_by_event(&$object, $filename, $from_ip, $from_user_agent, $date)
	{
		global $user, $langs;

		$langs->load('pdfdownloadnotifier@pdfdownloadnotifier');
		$title = $langs->trans('PdfDownloadNotifierEventTitle');
		$msg = $langs->trans('PdfDownloadNotifierEventMessage', $filename, $from_ip, $from_user_agent, dol_print_date($date, 'standard'));

		$socid = 0;
		$result = $object->fetch_thirdparty();
		if ($result > 0) {
			$socid = $object->thirdparty->id;
		}

		return $this->_addAction('AC_PDN_NOTIF', $socid, $object->id, $object->element, $title, $msg, $user);
	}

	/**
	 *  Get emails list of all the assigned to the request
	 *
	 * @param   string      $name       Name of the user
	 * @param   string      $name       Address email
	 * @return  string                  Formatted email (RFC 2822: "Name firstname <email>" or "email" or "<email>")
	 */
	protected function _formatEmail($name, $email)
	{
		if (!preg_match('/<|>/i', $email) && !empty($name)) {
			$email = str_replace(array('<', '>'), '', $name) . ' <' . $email . '>';
		}

		return $email;
	}

	/**
	 *  Add action
	 *
	 * @param   string      $type_code      Type code of the action
	 * @param   int         $socid          Company ID
	 * @param   int         $fk_element     Element linked ID
	 * @param   string      $elementtype    Element linked Type
	 * @param   string      $title          Title of the action
	 * @param   string      $msg            Message of the action
	 * @param   User        $user           User that modifies
	 * @return  int                         <0 if KO, >0 if OK
	 */
	protected function _addAction($type_code, $socid, $fk_element, $elementtype, $title, $msg, $user)
	{
		dol_syslog(__METHOD__ . " type_code=" . $type_code . " socid=" . $socid . " fk_element=" . $fk_element . " title=" . $title . " msg=" . $msg . " user_id=" . $user->id, LOG_DEBUG);
		$now = dol_now();
		// Insertion action
		require_once(DOL_DOCUMENT_ROOT . '/comm/action/class/actioncomm.class.php');
		$actioncomm = new ActionComm($this->db);
		$actioncomm->type_code = $type_code;
		$actioncomm->label = $title;
		$actioncomm->note = $msg;
		$actioncomm->datep = $now;
		$actioncomm->datef = $now;
		$actioncomm->durationp = 0;
		$actioncomm->punctual = 1;
		$actioncomm->percentage = -1; // Not applicable
		$actioncomm->contactid = 0;
		$actioncomm->socid = $socid;
		$actioncomm->author = $user; // User saving action
		// $actioncomm->usertodo = $user; // User affected to action
		$actioncomm->userdone = $user; // User doing action
		$actioncomm->fk_element = $fk_element;
		$actioncomm->elementtype = $elementtype;
		$actioncomm->userownerid = $user->id;

		$result = $actioncomm->create($user); // User qui saisit l'action
		if ($result < 0) {
			$this->error = $actioncomm->error;
			$this->errors = $actioncomm->errors;
			dol_syslog(__METHOD__ . " Errors: " . $actioncomm->errorsToString(), LOG_ERR);
		}

		return $result;
	}
}